package cn.reghao.tnb.user.app.db.mapper;

import cn.reghao.jutil.jdk.web.db.BaseMapper;
import cn.reghao.tnb.user.app.model.dto.ContactApplyInfo;
import cn.reghao.tnb.user.app.model.po.UserContactRecord;
import org.apache.ibatis.annotations.Mapper;

import java.util.List;

/**
 * @author reghao
 * @date 2022-04-18 15:32:55
 */
@Mapper
public interface UserContactRecordMapper extends BaseMapper<UserContactRecord> {
    void updateContactApply(int applyId, long appliedUser, int applyStatus);

    UserContactRecord findByUserIdAndFriendId(long applyUser, long appliedUser);

    UserContactRecord findByApplyId(int applyId, long appliedUser);
    int countNewApply(long appliedUser);
    List<ContactApplyInfo> findByNewApply(long appliedUser);
}
